@@ -27,29 +27,30 @@
 #include <qplatformdefs.h> //for readlink
 #include <assert.h>
 
-#include <QtCore/QCache>
-#include <QtCore/QFileInfo>
-#include <QtCore/QDir>
-#include <QtCore/QBuffer>
-#include <QtCore/QDataStream>
-#include <QtCore/QByteArray>
-#include <QtCore/QStringBuilder> // % operator for QString
+#include <QCache>
+#include <QFileInfo>
+#include <QDir>
+#include <QBuffer>
+#include <QDataStream>
+#include <QByteArray>
+#include <QStringBuilder> // % operator for QString
 #include <QElapsedTimer>
-#include <QGuiApplication>
 #include <QIcon>
 #include <QImage>
 #include <QMovie>
 #include <QPainter>
 #include <QPixmap>
 #include <QPixmapCache>
+#include <QGuiApplication>
 
 // kdecore
 #include <kconfiggroup.h>
 #include <kshareddatacache.h>
 #include <ksharedconfig.h>
-#include <QtDBus/QDBusConnection>
+#ifdef QT_DBUS_LIB
+#include <QDBusConnection>
 #include <QDBusMessage>
-#include <QSvgRenderer>
+#endif
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
 #include <QCryptographicHash>
@@ -156,6 +157,9 @@ public:
     KIconThemeNode(KIconTheme *_theme);
     ~KIconThemeNode();
 
+    KIconThemeNode(const KIconThemeNode &) = delete;
+    KIconThemeNode &operator=(const KIconThemeNode &) = delete;
+
     void queryIcons(QStringList *lst, int size, KIconLoader::Context context) const;
     void queryIconsByContext(QStringList *lst, int size, KIconLoader::Context context) const;
     QString findIcon(const QString &name, int size, KIconLoader::MatchType match) const;
@@ -208,8 +212,6 @@ class KIconLoaderPrivate
 public:
     KIconLoaderPrivate(KIconLoader *q)
         : q(q)
-        , mpGroups(nullptr)
-        , mIconCache(nullptr)
     {
     }
 
@@ -250,7 +252,7 @@ public:
      * tries to find an icon with the name. It tries some extension and
      * match strategies
      */
-    QString findMatchingIcon(const QString &name, int size) const;
+    QString findMatchingIcon(const QString &name, int size, qreal scale) const;
 
     /**
      * @internal
@@ -258,7 +260,7 @@ public:
      * This is one layer above findMatchingIcon -- it also implements generic fallbacks
      * such as generic icons for mimetypes.
      */
-    QString findMatchingIconWithGenericFallbacks(const QString &name, int size) const;
+    QString findMatchingIconWithGenericFallbacks(const QString &name, int size, qreal scale) const;
 
     /**
      * @internal
@@ -298,7 +300,7 @@ public:
      * @internal
      * return the path for the unknown icon in that size
      */
-    QString unknownIconPath(int size) const;
+    QString unknownIconPath(int size, qreal scale) const;
 
     /**
      * Checks if name ends in one of the supported icon formats (i.e. .png)
@@ -320,7 +322,7 @@ public:
      * icon metadata. Ensure the metadata is normalized first.
      */
     QString makeCacheKey(const QString &name, KIconLoader::Group group, const QStringList &overlays,
-                         int size, int state) const;
+                         int size, qreal scale, int state) const;
 
     /**
      * @internal
@@ -337,7 +339,7 @@ public:
      * @p size is only used for scalable images, but if non-zero non-scalable
      * images will be resized anyways.
      */
-    QImage createIconImage(const QString &path, int size = 0, KIconLoader::States state = KIconLoader::DefaultState);
+    QImage createIconImage(const QString &path, int size = 0, qreal scale = 1.0, KIconLoader::States state = KIconLoader::DefaultState);
 
     /**
      * @internal
@@ -375,14 +377,14 @@ public:
     KIconLoader *const q;
 
     QStringList mThemesInTree;
-    KIconGroup *mpGroups;
-    KIconThemeNode *mpThemeRoot;
+    KIconGroup *mpGroups = nullptr;
+    KIconThemeNode *mpThemeRoot = nullptr;
     QStringList searchPaths;
     KIconEffect mpEffect;
     QList<KIconThemeNode *> links;
 
     // This shares the icons across all processes
-    KSharedDataCache *mIconCache;
+    KSharedDataCache *mIconCache = nullptr;
 
     // This caches rendered QPixmaps in just this process.
     QCache<QString, PixmapWithPath> mPixmapCache;
@@ -416,15 +418,19 @@ public:
             parseGenericIconsFiles(file);
         }
 
+#ifdef QT_DBUS_LIB
         QDBusConnection::sessionBus().connect(QString(), QStringLiteral("/KIconLoader"), QStringLiteral("org.kde.KIconLoader"),
                                               QStringLiteral("iconChanged"), this, SIGNAL(iconChanged(int)));
+#endif
     }
 
     void emitChange(KIconLoader::Group group)
     {
+#ifdef QT_DBUS_LIB
         QDBusMessage message = QDBusMessage::createSignal(QStringLiteral("/KIconLoader"), QStringLiteral("org.kde.KIconLoader"), QStringLiteral("iconChanged"));
         message.setArguments(QList<QVariant>() << int(group));
         QDBusConnection::sessionBus().send(message);
+#endif
     }
 
     QString genericIconFor(const QString &icon) const
@@ -496,7 +502,7 @@ void KIconLoaderPrivate::drawOverlays(const KIconLoader *iconLoader, KIconLoader
     QPainter painter(&pix);
 
     int count = 0;
-    foreach (const QString &overlay, overlays) {
+    for (const QString &overlay : overlays) {
         // Ensure empty strings fill up a emblem spot
         // Needed when you have several emblems to ensure they're always painted
         // at the same place, even if one is not here
@@ -516,27 +522,27 @@ void KIconLoaderPrivate::drawOverlays(const KIconLoader *iconLoader, KIconLoader
 
         // match the emblem's devicePixelRatio to the original pixmap's
         pixmap.setDevicePixelRatio(pix.devicePixelRatio());
-        const int margin = 2 * pixmap.devicePixelRatio();
+        const int margin = pixmap.devicePixelRatio() * 0.05 * iconSize;
 
         QPoint startPoint;
         switch (count) {
         case 0:
-            // bottom left corner
-            startPoint = QPoint(margin, height - overlaySize - margin);
-            break;
-        case 1:
             // bottom right corner
             startPoint = QPoint(width - overlaySize - margin,
                                 height - overlaySize - margin);
             break;
-        case 2:
-            // top right corner
-            startPoint = QPoint(width - overlaySize - margin, margin);
+        case 1:
+            // bottom left corner
+            startPoint = QPoint(margin, height - overlaySize - margin);
             break;
-        case 3:
+        case 2:
             // top left corner
             startPoint = QPoint(margin, margin);
             break;
+        case 3:
+            // top right corner
+            startPoint = QPoint(width - overlaySize - margin, margin);
+            break;
         }
 
         startPoint /= pix.devicePixelRatio();
@@ -612,7 +618,7 @@ void KIconLoaderPrivate::init(const QString &_appname, const QStringList &extraS
     // loading config and default sizes
     initIconThemes();
     KIconTheme *defaultSizesTheme = links.empty() ? nullptr : links.first()->theme;
-    mpGroups = new KIconGroup[(int) KIconLoader::LastGroup];
+    mpGroups = new KIconGroup[static_cast<int>(KIconLoader::LastGroup)];
     for (KIconLoader::Group i = KIconLoader::FirstGroup; i < KIconLoader::LastGroup; ++i) {
         if (groups[i] == nullptr) {
             break;
@@ -857,19 +863,21 @@ void KIconLoaderPrivate::normalizeIconMetadata(KIconLoader::Group &group, int &s
 }
 
 QString KIconLoaderPrivate::makeCacheKey(const QString &name, KIconLoader::Group group,
-        const QStringList &overlays, int size, int state) const
+        const QStringList &overlays, int size, qreal scale, int state) const
 {
     // The KSharedDataCache is shared so add some namespacing. The following code
     // uses QStringBuilder (new in Qt 4.6)
 
     return (group == KIconLoader::User
-            ? QLatin1Literal("$kicou_")
-            : QLatin1Literal("$kico_"))
+            ? QLatin1String("$kicou_")
+            : QLatin1String("$kico_"))
            % name
            % QLatin1Char('_')
            % QString::number(size)
+           % QLatin1Char('@')
+           % QString::number(scale, 'f', 1)
            % QLatin1Char('_')
-           % overlays.join(QStringLiteral("_"))
+           % overlays.join(QLatin1Char('_'))
            % (group >= 0 ? mpEffect.fingerprint(group, state)
               : NULL_EFFECT_FINGERPRINT())
            % QLatin1Char('_')
@@ -927,28 +935,28 @@ QByteArray KIconLoaderPrivate::processSvg(const QString &path, KIconLoader::Stat
     return processedContents;
 }
 
-QImage KIconLoaderPrivate::createIconImage(const QString &path, int size, KIconLoader::States state)
+QImage KIconLoaderPrivate::createIconImage(const QString &path, int size, qreal scale, KIconLoader::States state)
 {
     //TODO: metadata in the theme to make it do this only if explicitly supported?
-    QScopedPointer<QImageReader> reader;
+    QImageReader reader;
     QBuffer buffer;
 
     if (q->theme()->followsColorScheme() && (path.endsWith(QLatin1String("svg")) || path.endsWith(QLatin1String("svgz")))) {
         buffer.setData(processSvg(path, state));
-        reader.reset(new QImageReader(&buffer));
+        reader.setDevice(&buffer);
     } else {
-        reader.reset(new QImageReader(path));
+        reader.setFileName(path);
     }
 
-    if (!reader->canRead()) {
+    if (!reader.canRead()) {
         return QImage();
     }
 
     if (size != 0) {
-        reader->setScaledSize(QSize(size, size));
+        reader.setScaledSize(QSize(size * scale, size * scale));
     }
 
-    return reader->read();
+    return reader.read();
 }
 
 void KIconLoaderPrivate::insertCachedPixmapWithPath(
@@ -1037,21 +1045,21 @@ bool KIconLoaderPrivate::findCachedPixmapWithPath(const QString &key, QPixmap &d
     return false;
 }
 
-QString KIconLoaderPrivate::findMatchingIconWithGenericFallbacks(const QString &name, int size) const
+QString KIconLoaderPrivate::findMatchingIconWithGenericFallbacks(const QString &name, int size, qreal scale) const
 {
-    QString path = findMatchingIcon(name, size);
+    QString path = findMatchingIcon(name, size, scale);
     if (!path.isEmpty()) {
         return path;
     }
 
     const QString genericIcon = s_globalData()->genericIconFor(name);
     if (!genericIcon.isEmpty()) {
-        path = findMatchingIcon(genericIcon, size);
+        path = findMatchingIcon(genericIcon, size, scale);
     }
     return path;
 }
 
-QString KIconLoaderPrivate::findMatchingIcon(const QString &name, int size) const
+QString KIconLoaderPrivate::findMatchingIcon(const QString &name, int size, qreal scale) const
 {
     const_cast<KIconLoaderPrivate *>(this)->initIconThemes();
 
@@ -1070,8 +1078,8 @@ QString KIconLoaderPrivate::findMatchingIcon(const QString &name, int size) cons
     // Once everone uses that to look up mimetype icons, we can kill the fallback code
     // from this method.
 
-    foreach (KIconThemeNode *themeNode, links) {
-        const QString path = themeNode->theme->iconPathByName(name, size, KIconLoader::MatchBest);
+    for (KIconThemeNode *themeNode : qAsConst(links)) {
+        const QString path = themeNode->theme->iconPathByName(name, size, KIconLoader::MatchBest, scale);
         if (!path.isEmpty()) {
             return path;
         }
@@ -1082,7 +1090,7 @@ QString KIconLoaderPrivate::findMatchingIcon(const QString &name, int size) cons
     }
     bool genericFallback = false;
     QString path;
-    foreach (KIconThemeNode *themeNode, links) {
+    for (KIconThemeNode *themeNode : qAsConst(links)) {
         QString currentName = name;
 
         while (!currentName.isEmpty()) {
@@ -1121,7 +1129,7 @@ QString KIconLoaderPrivate::findMatchingIcon(const QString &name, int size) cons
             }
 
             //qCDebug(KICONTHEMES) << "Looking up" << currentName;
-            path = themeNode->theme->iconPathByName(currentName, size, KIconLoader::MatchBest);
+            path = themeNode->theme->iconPathByName(currentName, size, KIconLoader::MatchBest, scale);
             if (!path.isEmpty()) {
                 return path;
             }
@@ -1130,11 +1138,11 @@ QString KIconLoaderPrivate::findMatchingIcon(const QString &name, int size) cons
     return path;
 }
 
-inline QString KIconLoaderPrivate::unknownIconPath(int size) const
+inline QString KIconLoaderPrivate::unknownIconPath(int size, qreal scale) const
 {
-    QString path = findMatchingIcon(QStringLiteral("unknown"), size);
+    QString path = findMatchingIcon(QStringLiteral("unknown"), size, scale);
     if (path.isEmpty()) {
-        qCDebug(KICONTHEMES) << "Warning: could not find \"unknown\" icon for size" << size;
+        qCDebug(KICONTHEMES) << "Warning: could not find \"unknown\" icon for size" << size << "at scale" << scale;
         return QString();
     }
     return path;
@@ -1142,7 +1150,7 @@ inline QString KIconLoaderPrivate::unknownIconPath(int size) const
 
 QString KIconLoaderPrivate::locate(const QString &fileName)
 {
-    Q_FOREACH (const QString &dir, searchPaths) {
+    for (const QString &dir : qAsConst(searchPaths)) {
         const QString path = dir + QLatin1Char('/') + fileName;
         if (QDir(dir).isAbsolute()) {
             if (QFileInfo::exists(path)) {
@@ -1162,6 +1170,12 @@ QString KIconLoaderPrivate::locate(const QString &fileName)
 
 QString KIconLoader::iconPath(const QString &_name, int group_or_size,
                               bool canReturnNull) const
+{
+    return iconPath(_name, group_or_size, canReturnNull, 1 /*scale*/);
+}
+
+QString KIconLoader::iconPath(const QString &_name, int group_or_size,
+                              bool canReturnNull, qreal scale) const
 {
     if (!d->initIconThemes()) {
         return QString();
@@ -1205,11 +1219,11 @@ QString KIconLoader::iconPath(const QString &_name, int group_or_size,
         if (canReturnNull) {
             return QString();
         } else {
-            return d->unknownIconPath(size);
+            return d->unknownIconPath(size, scale);
         }
     }
 
-    path = d->findMatchingIconWithGenericFallbacks(name, size);
+    path = d->findMatchingIconWithGenericFallbacks(name, size, scale);
 
     if (path.isEmpty()) {
         // Try "User" group too.
@@ -1218,7 +1232,7 @@ QString KIconLoader::iconPath(const QString &_name, int group_or_size,
             return path;
         }
 
-        return d->unknownIconPath(size);
+        return d->unknownIconPath(size, scale);
     }
     return path;
 }
@@ -1247,9 +1261,18 @@ QPixmap KIconLoader::loadMimeTypeIcon(const QString &_iconName, KIconLoader::Gro
     return pixmap;
 }
 
+
+
 QPixmap KIconLoader::loadIcon(const QString &_name, KIconLoader::Group group, int size,
                               int state, const QStringList &overlays,
                               QString *path_store, bool canReturnNull) const
+{
+return loadScaledIcon(_name, group, 1.0 /*scale*/, size, state, overlays, path_store, canReturnNull);
+}
+
+QPixmap KIconLoader::loadScaledIcon(const QString &_name, KIconLoader::Group group, qreal scale,
+                                    int size, int state, const QStringList &overlays,
+                                    QString *path_store, bool canReturnNull) const
 {
     QString name = _name;
     bool favIconOverlay = false;
@@ -1289,12 +1312,15 @@ QPixmap KIconLoader::loadIcon(const QString &_name, KIconLoader::Group group, in
     d->normalizeIconMetadata(group, size, state);
 
     // See if the image is already cached.
-    QString key = d->makeCacheKey(name, group, overlays, size, state);
+    QString key = d->makeCacheKey(name, group, overlays, size, scale, state);
     QPixmap pix;
+
     bool iconWasUnknown = false;
     QString path;
 
     if (d->findCachedPixmapWithPath(key, pix, path)) {
+        pix.setDevicePixelRatio(scale);
+
         if (path_store) {
             *path_store = path;
         }
@@ -1323,7 +1349,7 @@ QPixmap KIconLoader::loadIcon(const QString &_name, KIconLoader::Group group, in
         if (absolutePath && !favIconOverlay) {
             path = name;
         } else {
-            path = d->findMatchingIconWithGenericFallbacks(favIconOverlay ? QStringLiteral("text-html") : name, size);
+            path = d->findMatchingIconWithGenericFallbacks(favIconOverlay ? QStringLiteral("text-html") : name, size, scale);
         }
     }
 
@@ -1336,16 +1362,16 @@ QPixmap KIconLoader::loadIcon(const QString &_name, KIconLoader::Group group, in
     // Still can't find it? Use "unknown" if we can't return null.
     // We keep going in the function so we can ensure this result gets cached.
     if (path.isEmpty() && !canReturnNull) {
-        path = d->unknownIconPath(size);
+        path = d->unknownIconPath(size, scale);
         iconWasUnknown = true;
     }
 
     QImage img;
     if (!path.isEmpty()) {
-        img = d->createIconImage(path, size, (KIconLoader::States)state);
+        img = d->createIconImage(path, size, scale, static_cast<KIconLoader::States>(state));
     }
 
-    if (group >= 0) {
+    if (group >= 0 && group < KIconLoader::LastGroup) {
         img = d->mpEffect.apply(img, group, state);
     }
 
@@ -1437,7 +1463,7 @@ QString KIconLoader::moviePath(const QString &name, KIconLoader::Group group, in
 
         QString path;
 
-        foreach (KIconThemeNode *themeNode, d->links) {
+        for (KIconThemeNode *themeNode : qAsConst(d->links)) {
             path = themeNode->theme->iconPath(file, size, KIconLoader::MatchExact);
             if (!path.isEmpty()) {
                 break;
@@ -1445,7 +1471,7 @@ QString KIconLoader::moviePath(const QString &name, KIconLoader::Group group, in
         }
 
         if (path.isEmpty()) {
-            foreach (KIconThemeNode *themeNode, d->links) {
+            for (KIconThemeNode *themeNode : qAsConst(d->links)) {
                 path = themeNode->theme->iconPath(file, size, KIconLoader::MatchBest);
                 if (!path.isEmpty()) {
                     break;
@@ -1484,7 +1510,7 @@ QStringList KIconLoader::loadAnimated(const QString &name, KIconLoader::Group gr
         if (size == 0) {
             size = d->mpGroups[group].size;
         }
-        file = d->findMatchingIcon(file, size);
+        file = d->findMatchingIcon(file, size, 1); // FIXME scale
     }
     if (file.isEmpty()) {
         return lst;
@@ -1496,7 +1522,8 @@ QStringList KIconLoader::loadAnimated(const QString &name, KIconLoader::Group gr
         return lst;
     }
 
-    foreach (const QString &entry, dir.entryList()) {
+    const auto entryList = dir.entryList();
+    for (const QString &entry : entryList) {
         if (!(entry.leftRef(4)).toUInt()) {
             continue;
         }
@@ -1558,7 +1585,7 @@ QStringList KIconLoader::queryIconsByContext(int group_or_size,
         size = -group_or_size;
     }
 
-    foreach (KIconThemeNode *themeNode, d->links) {
+    for (KIconThemeNode *themeNode : qAsConst(d->links)) {
         themeNode->queryIconsByContext(&result, size, context);
     }
 
@@ -1599,7 +1626,7 @@ QStringList KIconLoader::queryIcons(int group_or_size, KIconLoader::Context cont
         size = -group_or_size;
     }
 
-    foreach (KIconThemeNode *themeNode, d->links) {
+    for (KIconThemeNode *themeNode : qAsConst(d->links)) {
         themeNode->queryIcons(&result, size, context);
     }
 
@@ -1628,7 +1655,7 @@ bool KIconLoader::hasContext(KIconLoader::Context context) const
 {
     d->initIconThemes();
 
-    foreach (KIconThemeNode *themeNode, d->links)
+    for (KIconThemeNode *themeNode : qAsConst(d->links))
         if (themeNode->theme->hasContext(context)) {
             return true;
         }
@@ -1672,6 +1699,7 @@ QIcon KIconLoader::loadIconSet(const QString &name, KIconLoader::Group g, int s,
 
 // Easy access functions
 
+#ifndef KICONTHEMES_NO_DEPRECATED
 QPixmap DesktopIcon(const QString &name, int force_size, int state, const QStringList &overlays)
 {
     KIconLoader *loader = KIconLoader::global();
@@ -1679,13 +1707,11 @@ QPixmap DesktopIcon(const QString &name, int force_size, int state, const QStrin
 }
 
 // deprecated
-#ifndef KICONTHEMES_NO_DEPRECATED
 QIcon DesktopIconSet(const QString &name, int force_size)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIconSet(name, KIconLoader::Desktop, force_size);
 }
-#endif
 
 QPixmap BarIcon(const QString &name, int force_size, int state, const QStringList &overlays)
 {
@@ -1694,13 +1720,11 @@ QPixmap BarIcon(const QString &name, int force_size, int state, const QStringLis
 }
 
 // deprecated
-#ifndef KICONTHEMES_NO_DEPRECATED
 QIcon BarIconSet(const QString &name, int force_size)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIconSet(name, KIconLoader::Toolbar, force_size);
 }
-#endif
 
 QPixmap SmallIcon(const QString &name, int force_size, int state, const QStringList &overlays)
 {
@@ -1709,13 +1733,11 @@ QPixmap SmallIcon(const QString &name, int force_size, int state, const QStringL
 }
 
 // deprecated
-#ifndef KICONTHEMES_NO_DEPRECATED
 QIcon SmallIconSet(const QString &name, int force_size)
 {
     KIconLoader *loader = KIconLoader::global();
     return loader->loadIconSet(name, KIconLoader::Small, force_size);
 }
-#endif
 
 QPixmap MainBarIcon(const QString &name, int force_size, int state, const QStringList &overlays)
 {
@@ -1724,7 +1746,6 @@ QPixmap MainBarIcon(const QString &name, int force_size, int state, const QStrin
 }
 
 // deprecated
-#ifndef KICONTHEMES_NO_DEPRECATED
 QIcon MainBarIconSet(const QString &name, int force_size)
 {
     KIconLoader *loader = KIconLoader::global();
@@ -1756,7 +1777,7 @@ int IconSize(KIconLoader::Group group)
 QPixmap KIconLoader::unknown()
 {
     QPixmap pix;
-    if (QPixmapCache::find(QStringLiteral("unknown"), pix)) { //krazy:exclude=iconnames
+    if (QPixmapCache::find(QStringLiteral("unknown"), &pix)) { //krazy:exclude=iconnames
         return pix;
     }
 
